// This code has been placed in the public domain.

/////////////////////////////////////////////////////////////////////////////////
function Color(inRed, inGreen, inBlue, inAlpha, inTransparent)
{
	this.r = inRed;
	this.g = inGreen;
	this.b = inBlue;
	this.a = inAlpha;
	if (null == this.a)
		this.a = 1.0;
	this.transparent = inTransparent;
	if (null == this.transparent)
		this.transparent = false;
} // Color

Color.prototype.toString = function ()
{
	if (true == this.transparent)
		return "transparent";
		
	var strColor = "rgb(" + this.r + ", " + this.g + ", " + this.b + ")";
	
	return strColor;
} // toString

Color.prototype.toOpacityString = function ()
{
	var strOpacity = "" + this.a;
	
	return strOpacity;
} // toOpacityString

Color.prototype.toFilterString = function ()
{
	var fFilter = this.a * 100;
	var strFilter = "filter:alpha(opacity=" + fFilter + ")";
	
	return strFilter;
} // toFilterString

Color.prototype.fromString = function (inColorString)
{
	if ("transparent" == inColorString)
	{
		this.r = 0;
		this.g = 0;
		this.b = 0;
		this.a = 0;
		this.transparent = true;
		return;
	} // if
	
	var strColor = inColorString.replace(/rgb\(/, "");
	
	strColor = strColor.replace(/\)/g, "");
	strColor = strColor.replace(/ /g, "");
	
	var aRGB = strColor.split(",");
	
	this.r = aRGB[0];
	this.g = aRGB[1];
	this.b = aRGB[2];
} // fromString

Color.prototype.fromOpacityString = function (inString)
{
	if (null == inString)
		return;
	
	this.a = inString;
} // fromOpacityString

Color.prototype.fromFilterString = function (inString)
{
	if (null == inString)
		return;
	
	var aFilter = inString.split("=");
	var strFilter = aFilter[1].replace(/\)/g, "");

	strFilter = strFilter.replace(/ /g, "");
	
	this.a = Math.round((parseFloat(strFilter) / 100));
} // fromFilterString

Color.prototype.isEqual = function (inColor)
{
	if (this.r != inColor.r)
		return false;
	if (this.g != inColor.g)
		return false;
	if (this.b != inColor.b)
		return false;
	if (this.a != inColor.a)
		return false;
	if (this.transparent != inColor.transparent)
		return false;
		
	return true;
} // isEqual

 Color.prototype.invert = function()
{
    this.r = 255 - this.r;
    this.g = 255 - this.g;
    this.b = 255 - this.b;
} // invert

/////////////////////////////////////////////////////////////////////////////////
function MouseClickInfo(inMouseClickFunction, inMouseClickColor, inMouseClickFillColor, inMouseDownFunction, inMouseDownColor, inMouseDownFillColor, inMouseUpFunction, inMouseUpColor, inMouseUpFillColor, inMouseDoubleClickFunction, inMouseDoubleClickColor, inMouseDoubleClickFillColor)
{
	this.mouseClickFunction = inMouseClickFunction;
	this.mouseClickColor = inMouseClickColor;
	this.mouseClickFillColor = inMouseClickFillColor;
	this.mouseDownFunction = inMouseDownFunction;
	this.mouseDownColor = inMouseDownColor;
	this.mouseDownFillColor = inMouseDownFillColor;
	this.mouseUpFunction = inMouseUpFunction;
	this.mouseUpColor = inMouseUpColor;
	this.mouseUpFillColor = inMouseUpFillColor;
	this.mouseDoubleClickFunction = inMouseDoubleClickFunction;
	this.mouseDoubleClickColor = inMouseDoubleClickColor;
	this.mouseDoubleClickFillColor = inMouseDoubleClickFillColor;
} // MouseClickInfo

function MouseMoveInfo(inMouseOverFunction, inMouseOverColor, inMouseOverFillColor, inMouseOutFunction, inMouseOutColor, inMouseOutFillColor)
{
	this.mouseOverFunction = inMouseOverFunction;
	this.mouseOverColor = inMouseOverColor;
	this.mouseOverFillColor = inMouseOverFillColor;
	this.mouseOutFunction = inMouseOutFunction;
	this.mouseOutColor = inMouseOutColor;
	this.mouseOutFillColor = inMouseOutFillColor;
} // MouseMoveInfo

function EventInfo(inMouseClickInfo, inMouseMoveInfo)
{
	this.mouseClickInfo = inMouseClickInfo;
	this.mouseMoveInfo = inMouseMoveInfo;
} // EventInfo

/////////////////////////////////////////////////////////////////////////////////
function Shape()
{
	this.x = 0;
	this.y = 0;
	this.width = 0;
	this.height = 0;
	this.visible = true;
	this.children = new Array();
	this.drawColor = null;
	this.fillColor = null;
} // Shape
Shape.prototype.DEG2RAD = 3.14159/180;

Shape.prototype.addShape = function (inShape)
{
 	this.children.push(inShape);
} // addShape

Shape.prototype.removeShape = function (inShape)
{
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		var oShape = this.children[iLoop1];
		
		if (false == oShape.equals(inShape))
			continue; // for
		
		oShape.view = null;
		this.children.splice(iLoop1, 1);
	} // for
} // removeShape

Shape.prototype.getArea = function ()
{
	return this.width * this.height;
} // getArea

Shape.prototype.containsPoint = function (inPoint)
{
	if (inPoint.x < this.x)
		return false;
 	if (inPoint.x > this.x + this.width)
		return false;
 	if (inPoint.y < this.y)
		return false;
 	if (inPoint.y > this.y + this.height)
		return false;
		
 	return true;
} // containsPoint

Shape.prototype.isEqual = function (inShape)
{
	if (this.x != inShape.x)
		return false;
	if (this.y != inShape.y)
		return false;
	if (this.width != inShape.width)
		return false;
	if (this.height != inShape.height)
		return false;
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		if (false == this.children[iLoop1].isEqual(inShape.children[iLoop1]))
			return false;
	} // for
		
	return true;
} // isEqual

Shape.prototype.draw = function (inView)
{
} // draw

Shape.prototype.drawChildren = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		var oChildShape = this.children[iLoop1];
		
		oChildShape.draw(inView);
		oChildShape.drawChildren(inView);
	} // for
} // drawChildren

Shape.prototype.erase = function (inView)
{
	var oDrawColor = this.drawColor;
	var oEventInfo = this.eventInfo;
	
	this.drawColor = inView.backgroundColor;
	this.setEventInfo(inView.eventInfo);
	this.draw(inView, true);
	this.drawColor = oDrawColor;
	this.setEventInfo(oEventInfo);
} // erase

Shape.prototype.eraseChildren = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		var oChildShape = this.children[iLoop1];
		
		oChildShape.erase(inView);
		oChildShape.eraseChildren(inView);
	} // for
} // eraseChildren

Shape.prototype.fill = function (inView, inEraseFlag)
{
} // fill

Shape.prototype.fillChildren = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		var oChildShape = this.children[iLoop1];
		
		oChildShape.fill(inView);
		oChildShape.fillChildren(inView);
	} // for
} // fillChildren

Shape.prototype.empty = function (inView)
{
 	var oFillColor = this.drawColor;
	var oEventInfo = this.eventInfo;
	
	this.fillColor = inView.backgroundColor;
	this.setEventInfo(inView.eventInfo);
	this.fill(inView);
	this.fillColor = oFillColor;
	this.setEventInfo(oEventInfo);
} // empty

Shape.prototype.emptyChildren = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.children.length; iLoop1++)
	{
		var oChildShape = this.children[iLoop1];
		
		oChildShape.empty(inView);
		oChildShape.emptyChildren(inView);
	} // for
} // emptyChildren

Shape.prototype.setEventInfo = function (inEventInfo)
{
	this.eventInfo = inEventInfo;
} // setEventInfo

/////////////////////////////////////////////////////////////////////////////////
function Point(inX, inY, inDrawColor)
{
	this.x = inX;
	this.y = inY;
	this.width = 0;
	this.height = 0;
	this.parent = "Shape";
	this.drawColor = inDrawColor;
} // Point
Point.prototype = new Shape();

Point.prototype.getCopy = function ()
{
	var oPoint = new Point(this.x, this.y, this.drawColor);
	 
	oPoint.setEventInfo(this.eventInfo);
	return oPoint;
} // getCopy

Point.prototype.add = function (inPt)
{
	var oPt = this.getCopy();
	
	oPt.x += inPt.x;
	oPt.y += inPt.y;
	return oPt;
} // add

Point.prototype.subtract = function (inPt)
{
	var oPt = this.getCopy();
	
	oPt.x -= inPt.x;
	oPt.y -= inPt.y;
	return oPt;
} // subtract

Point.prototype.getDistanceSquared = function (inPoint)
{
	var fXDistance = this.x - inPoint.x;
	var fYDistance = this.y - inPoint.y;
	var fDistanceSquared = fXDistance * fXDistance + fYDistance * fYDistance;
	
	return fDistanceSquared;
} // getDistanceSquared

Point.prototype.average = function(inPoint)
{
	var oAveragePoint = new Point();
	
	oAveragePoint.x = (this.x + inPoint.x) * 0.5;
	oAveragePoint.y = (this.y + inPoint.y) * 0.5;
	
	return oAveragePoint;
} // average

Point.prototype.dotProduct = function(inPoint)
{
	return this.x * inPoint.x + this.y * inPoint.y;
} // dotProduct

Point.prototype.crossProduct = function(inPoint)
{
	return this.x * inPoint.y - this.y * inPoint.x;
} // crossProduct

Point.prototype.draw = function (inView)
{
	if (false == this.visible)
		return;
		
 	inView.setColor(this.x, this.y, this.drawColor);
	if (null != this.eventInfo)
		inView.setEvent(this.x, this.y, this.eventInfo, this);
} // draw

/////////////////////////////////////////////////////////////////////////////////
function Line(inStartPoint, inEndPoint, inDrawColor)
{
	this.start = inStartPoint;
	this.end = inEndPoint;
	this.x = Math.min(inStartPoint.x, inEndPoint.x);
	this.y = Math.min(inStartPoint.y, inEndPoint.y);
	this.width = Math.max(inStartPoint.x, inEndPoint.x) - Math.min(inStartPoint.x, inEndPoint.x);
	this.height = Math.max(inStartPoint.y, inEndPoint.y) - Math.min(inStartPoint.y, inEndPoint.y);
	this.drawColor = inDrawColor;
	
	var aPoints = [];
    var dx = inEndPoint.x - inStartPoint.x;
    var dy = inEndPoint.y - inStartPoint.y;
	var x0 = inStartPoint.x;
	var y0 = inStartPoint.y;
	var x1 = inEndPoint.x;
	var y1 = inEndPoint.y;
	var stepx;
	var stepy;

    aPoints.push(inStartPoint);

    if (dy < 0)
	{
		dy = -dy;
		stepy = -1;
	} // if
	else
	{
		stepy = 1;
	} // else
    if (dx < 0)
	{
		dx = -dx;
		stepx = -1;
	} // if
	else
	{
		stepx = 1;
	} // else
    dy <<= 1;                                                  // dy is now 2*dy
    dx <<= 1;                                                  // dx is now 2*dx

    if (dx > dy)
	{
        var fraction = dy - (dx >> 1);                         // same as 2*dy - dx
        
        while (x0 != x1)
		{
            if (fraction >= 0)
			{
                y0 += stepy;
                fraction -= dx;                                // same as fraction -= 2*dx
            } // if
            x0 += stepx;
            fraction += dy;                                    // same as fraction -= 2*dy
 			aPoints.push(new Point(x0, y0));
        } // while
    } // if
	else
	{
        var fraction = dx - (dy >> 1);
		
        while (y0 != y1)
		{
            if (fraction >= 0)
			{
                x0 += stepx;
                fraction -= dy;
            } // if
            y0 += stepy;
            fraction += dx;
 			aPoints.push(new Point(x0, y0));
         } // while
    } // if

	this.points = aPoints;
	this.coefficientA = inStartPoint.y - inEndPoint.y;
	this.coefficientB = inEndPoint.x - inStartPoint.x;
	this.coefficientC = -0.5 * (this.coefficientA * (inStartPoint.x + inEndPoint.x) + this.coefficientB * (inStartPoint.y + inEndPoint.y));
} // Line
Line.prototype = new Shape();

Line.prototype.getArea = function ()
{
	return 0;
} // getArea

Line.prototype.isEqual = function (inLine)
{
	if ((this.start.isEqual(inLine.start))
	&& (this.end.isEqual(inLine.end)))
		return true;
	if ((this.start.isEqual(inLine.end))
	&& (this.end.isEqual(inLine.start)))
		return true;
		
	return false;
} // isEqual

Line.prototype.containsPoint = function (inPoint)
{
	var aPoints = this.points;
	
	for (var iLoop1 = 0; iLoop1 < aPoints.length; iLoop1++)
	{
		var oPoint = aPoints[iLoop1];
		
		if (true == oPoint.isEqual(inPoint))
			return true;
	} // for
	return false;
} // containsPoint

Line.prototype.getLength = function ()
{
	var fDX = this.points[0].x - this.points[1].x;
	var fDY = this.points[0].y - this.points[1].y;
	var fDZ = this.points[0].z - this.points[1].z;
	var fLength = Math.sqrt((fDX * fDX) + (fDY * fDY) + (fDZ * fDZ));
	
	return fLength;
} // getLength

Line.prototype.getLengthSquared = function ()
{
	var fDX = this.points[0].x - this.points[1].x;
	var fDY = this.points[0].y - this.points[1].y;
	var fDZ = this.points[0].z - this.points[1].z;
	var fLength = (fDX * fDX) + (fDY * fDY) + (fDZ * fDZ);
	
	return fLength;
} // getLengthSquared

Line.prototype.draw = function (inView)
{
	if (false == this.visible)
		return;
		
 	var aPoints = this.points;
	
	for (var iLoop1 = 0; iLoop1 < aPoints.length; iLoop1++)
	{
		var oPoint = aPoints[iLoop1];
		
		inView.setColor(oPoint.x, oPoint.y, this.drawColor);
		if (null != this.eventInfo)
			inView.setEvent(oPoint.x, oPoint.y, this.eventInfo, this);
	} // for
} // draw

/////////////////////////////////////////////////////////////////////////////////
function QuadraticCurve(inEndPoint1, inEndPoint2, inCenterPoint, inDrawColor, inSteps)
{
    this.endPoint1 = inEndPoint1;
    this.endPoint2 = inEndPoint2;
    this.centerPoint = inCenterPoint;
	this.drawColor = inDrawColor;
	this.lines = [];
	
	var aPoints = [];
	var fCount = 0;
    var iSteps = ((null == inSteps)? 100 : inSteps);
	var fDetailBias = 1 / iSteps;
			
	do
	{
		var fB1 = fCount * fCount;
		var fB2 = 2 * fCount * (1 - fCount);
		var fB3 = (1 - fCount) * (1 - fCount);
		var fEnd1XB1 = this.endPoint1.x * fB1;
		var fCenterXB2 = this.centerPoint.x * fB2;
		var fEnd2XB3 = this.endPoint2.x * fB3;
		var fEnd1YB1 = this.endPoint1.y * fB1;
		var fCenterYB2 = this.centerPoint.y * fB2;
		var fEnd2YB3 = this.endPoint2.y * fB3;
		var fX = fEnd1XB1 + fCenterXB2 + fEnd2XB3;
		var fY = fEnd1YB1 + fCenterYB2 + fEnd2YB3;
		// var fZ = fEnd1ZB1 + fCenterZB2 + fEnd2ZB3;
		var fPoint = new Point(Math.round(fX), Math.round(fY), inDrawColor);

		aPoints.push(fPoint);
		fCount += fDetailBias;
	}while (fCount <= 1);
	
	for (var iLoop1 = 0; iLoop1 < aPoints.length - 1; iLoop1++)
	{
		var oPoint1 = aPoints[iLoop1];
		var oPoint2 = aPoints[iLoop1 + 1];
		var oLine = new Line(oPoint1, oPoint2, inDrawColor);
		
		this.lines.push(oLine);
	} // for
} // QuadraticCurve
QuadraticCurve.prototype = new Shape();

QuadraticCurve.prototype.getArea = function ()
{
	return 0;
} // getArea

QuadraticCurve.prototype.isEqual = function (inQuadraticCurve)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oThisLine = this.lines[iLoop1];
		var oInLine = inQuadraticCurve.lines[iLoop1];
		
		if (false == oThisLine.isEqual(oInLine))
			return false;
	} // for
		
	return true;
} // isEqual

QuadraticCurve.prototype.containsPoint = function (inPoint)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oLine = this.lines[iLoop1];
		
		if (true == oLine.containsPoint(inPoint))
			return true;
	} // for
	return false;
} // containsPoint

QuadraticCurve.prototype.draw = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oLine = this.lines[iLoop1];
		
		oLine.drawColor = this.drawColor;
		oLine.setEventInfo(this.eventInfo);
		oLine.draw(inView);
	} // for
} // draw

/////////////////////////////////////////////////////////////////////////////////
function CubicCurve(inEndPoint1, inEndPoint2, inCenterPoint1, inCenterPoint2, inDrawColor, inSteps)
{
    this.endPoint1 = inEndPoint1;
    this.endPoint2 = inEndPoint2;
    this.centerPoint1 = inCenterPoint1;
    this.centerPoint2 = inCenterPoint2;
	this.drawColor = inDrawColor;
	this.lines = [];
	
	var aPoints = [];
	var fCount = 0;
    var iSteps = ((null == inSteps)? 100 : inSteps);
	var fDetailBias = 1 / iSteps; 
			
	do
	{
		var fB1 = fCount * fCount* fCount;
		var fB2 = 3 * (fCount * fCount) * (1 - fCount);
		var fB3 = 3 * fCount * ((1 - fCount) * (1 - fCount));
		var fB4 = (1 - fCount) * (1 - fCount) * (1 - fCount);
		var fEnd1XB1 = this.endPoint1.x * fB1;
		var fCenter1XB2 = this.centerPoint1.x * fB2;
		var fCenter2XB3 = this.centerPoint2.x * fB3;
		var fEnd2XB4 = this.endPoint2.x * fB4;
		var fEnd1YB1 = this.endPoint1.y * fB1;
		var fCenter1YB2 = this.centerPoint1.y * fB2;
		var fCenter2YB3 = this.centerPoint2.y * fB3;
		var fEnd2YB4 = this.endPoint2.y * fB4;
		var fX = fEnd1XB1 + fCenter1XB2 + fCenter2XB3 + fEnd2XB4;
		var fY = fEnd1YB1 + fCenter1YB2 + fCenter2YB3 + fEnd2YB4;
		// var fZ = fEnd1ZB1 + fCenter1ZB2 + fCenter2ZB3 + fEnd2ZB4;
		var fPoint = new Point(Math.round(fX), Math.round(fY), inDrawColor);

		aPoints.push(fPoint);
		fCount += fDetailBias;
	}while (fCount <= 1);
	
	for (var iLoop1 = 0; iLoop1 < aPoints.length - 1; iLoop1++)
	{
		var oPoint1 = aPoints[iLoop1];
		var oPoint2 = aPoints[iLoop1 + 1];
		var oLine = new Line(oPoint1, oPoint2, inDrawColor);
		
		this.lines.push(oLine);
	} // for
} // CubicCurve
CubicCurve.prototype = new Shape();

CubicCurve.prototype.getArea = function ()
{
	return 0;
} // getArea

CubicCurve.prototype.isEqual = function (inQuadraticCurve)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oThisLine = this.lines[iLoop1];
		var oInLine = inQuadraticCurve.lines[iLoop1];
		
		if (false == oThisLine.isEqual(oInLine))
			return false;
	} // for
		
	return true;
} // isEqual

CubicCurve.prototype.containsPoint = function (inPoint)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oLine = this.lines[iLoop1];
		
		if (true == oLine.containsPoint(inPoint))
			return true;
	} // for
	return false;
} // containsPoint

CubicCurve.prototype.draw = function (inView)
{
	for (var iLoop1 = 0; iLoop1 < this.lines.length; iLoop1++)
	{
		var oLine = this.lines[iLoop1];
		
		oLine.drawColor = this.drawColor;
		oLine.setEventInfo(this.eventInfo);
		oLine.draw(inView);
	} // for
} // draw

/////////////////////////////////////////////////////////////////////////////////
function Triangle(inPoint1, inPoint2, inPoint3, inDrawColor, inFillColor)
{
	this.point1 = inPoint1;
	this.point2 = inPoint2;
	this.point3 = inPoint3;
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
	
	var iMinX = Math.min(this.point1.x, this.point2.x);
	var iMaxX = Math.max(this.point1.x, this.point2.x);
	var iMinY = Math.min(this.point1.y, this.point2.y);
	var iMaxY = Math.max(this.point1.y, this.point2.y);
	
	iMinX = Math.min(iMinX, this.point3.x);
	iMaxX = Math.max(iMaxX, this.point3.x);
	iMinY = Math.min(iMinY, this.point3.y);
	iMaxY = Math.max(iMaxY, this.point3.y);
	
	this.x = iMinX;
	this.y = iMinY;
	this.width = iMaxX - iMinX;
	this.height = iMaxY - iMinY;
} // Triangle
Triangle.prototype = new Shape();

Triangle.prototype.getArea = function ()
{
	var oLine1 = new Line(this.point1, this.point2);
	var oLine2 = new Line(this.point2, this.point3);
	var oLine3 = new Line(this.point3, this.point1);
	var fLength1 = oLine1.getLength();
	var fLength2 = oLine2.getLength();
	var fLength3 = oLine3.getLength();
	var fHalfPerimeter = (fLength1 + fLength2 + fLength3) / 2;
	var fArea = fHalfPerimeter * (fLength1 - fHalfPerimeter) * (fLength2 - fHalfPerimeter) * (fLength3 - fHalfPerimeter);
	
	fArea = Math.abs(fArea);
	fArea = Math.sqrt(fArea);
	return fArea;
} // getArea

Triangle.prototype.containsPoint = function (inPoint)
{
	var oTriangle1 = new Triangle(inPoint, this.point1, this.point2);
	var oTriangle2 = new Triangle(inPoint, this.point2, this.point3);
	var oTriangle3 = new Triangle(inPoint, this.point1, this.point3);
	var fArea1 = oTriangle1.getArea();
	var fArea2 = oTriangle2.getArea();
	var fArea3 = oTriangle3.getArea();
	
	return ((fArea1 + fArea2 + fArea3) == this.getArea());
	/*
	 * Another way to do this. May be faster.
float ax, ay, bx, by, cx, cy, apx, apy, bpx, bpy, cpx, cpy;
float cCROSSap, bCROSScp, aCROSSbp;

ax = Cx - Bx;  ay = Cy - By;
bx = Ax - Cx;  by = Ay - Cy;
cx = Bx - Ax;  cy = By - Ay;
apx= Px - Ax;  apy= Py - Ay;
bpx= Px - Bx;  bpy= Py - By;
cpx= Px - Cx;  cpy= Py - Cy;

aCROSSbp = ax*bpy - ay*bpx;
cCROSSap = cx*apy - cy*apx;
bCROSScp = bx*cpy - by*cpx;

return ((aCROSSbp >= 0.0f) && (bCROSScp >= 0.0f) && (cCROSSap >= 0.0f));
	*/
} // containsPoint

Triangle.prototype.isEqual = function (inTriangle)
{
	var aPoints = [];
	var bPoint1Matched = false;
	var bPoint2Matched = false;
	var bPoint3Matched = false;
	
	aPoints.push(inTriangle.point1);
	aPoints.push(inTriangle.point2);
	aPoints.push(inTriangle.point3);
	
	for (var iLoop1 = 0; iLoop1 < aPoints.length; iLoop1++)
	{
		var oPoint = aPoints[iLoop1];
		
		if (false == this.point1.isEqual(oPoint))
			continue; // for
			
		bPoint1Matched = true;
		aPoints.splice(iLoop1, 1);
	} // for
	if (false == bPoint1Matched)
		return false;
	
	for (var iLoop2 = 0; iLoop2 < aPoints.length; iLoop2++)
	{
		var oPoint = aPoints[iLoop2];
		
		if (false == this.point2.isEqual(oPoint))
			continue; // for
			
		bPoint2Matched = true;
		aPoints.splice(iLoop2, 1);
	} // for
	if (false == bPoint2Matched)
		return false;
	
	for (var iLoop3 = 0; iLoop3 < aPoints.length; iLoop3++)
	{
		var oPoint = aPoints[iLoop3];
		
		if (false == this.point3.isEqual(oPoint))
			continue; // for
			
		bPoint3Matched = true;
		aPoints.splice(iLoop3, 1);
	} // for
	if (false == bPoint3Matched)
		return false;
		
	return true;
} // isEqual

Triangle.prototype.isClockwise = function ()
{
	var fDeterminant = (this.point2.x - this.point1.x) * (this.point3.y - this.point1.y) - (this.point3.x - this.point1.x) * (this.point2.y - this.point1.y);
 
    if (fDeterminant >= 0.0)
        return false;
    else
        return true;
} // isClockwise

Triangle.prototype.draw = function(inView)
{
	if (false == this.visible) 
		return;
	
	var oLine1 = new Line(this.point1, this.point2, this.drawColor);
	var oLine2 = new Line(this.point2, this.point3, this.drawColor);
	var oLine3 = new Line(this.point3, this.point1, this.drawColor);
	
	oLine1.setEventInfo(this.eventInfo);
	oLine2.setEventInfo(this.eventInfo);
	oLine3.setEventInfo(this.eventInfo);
	oLine1.draw(inView);
	oLine2.draw(inView);
	oLine3.draw(inView);
} // draw

Triangle.prototype.fill = function (inView)
{
	if (false == this.visible)
		return;
		
	var iRight = this.x + this.width;
	var iBottom = this.y + this.height;
	//a = y0 - y1
	//b = x1 - x0
	//c = x0 * y1 - x1 * y0
	var fA0 = this.point1.y - this.point2.y;
	var fB0 = this.point2.x - this.point1.x;
	var fC0 = this.point1.x * this.point2.y - this.point2.x * this.point1.y;
	var fA1 = this.point2.y - this.point3.y;
	var fB1 = this.point3.x - this.point2.x;
	var fC1 = this.point2.x * this.point3.y - this.point3.x * this.point2.y;
	var fA2 = this.point3.y - this.point1.y;
	var fB2 = this.point1.x - this.point3.x;
	var fC2 = this.point3.x * this.point1.y - this.point1.x * this.point3.y;
	var fArea = (fC0 + fC1 + fC2) * 0.5;
	var fFlip = 1;
	
	if (0.0 == fArea)
		return;
	else if (0.0 > fArea)
		fFlip = -1;
		
	for (var iXLoop = this.x; iXLoop <= iRight; iXLoop++)
	{
		for (var iYLoop = this.y; iYLoop <= iBottom; iYLoop++)
		{
			var fEdge0 = (fA0 * iXLoop + fB0 * iYLoop + fC0) * fFlip;
			var fEdge1 = (fA1 * iXLoop + fB1 * iYLoop + fC1) * fFlip;
			var fEdge2 = (fA2 * iXLoop + fB2 * iYLoop + fC2) * fFlip;
			
			if ((fEdge0 > 0)
			&& (fEdge1 > 0)
			&& (fEdge2 > 0))
			{
				inView.setColor(iXLoop, iYLoop, this.fillColor);
				if (null != this.eventInfo)
					inView.setEvent(iXLoop, iYLoop, this.eventInfo, this);
			} // if
		} // for
	} // for
} // fill

/////////////////////////////////////////////////////////////////////////////////
function Rect(inX, inY, inWidth, inHeight, inDrawColor, inFillColor)
{
	this.x = inX;
	this.y = inY;
	this.width = inWidth;
	this.height = inHeight;
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
} // Rect
Rect.prototype = new Shape();

Rect.prototype.draw = function (inView)
{
 	if (false == this.visible)
		return;
		
	var strColor = this.drawColor.toString();
	var iXLimit = this.x + this.width;
	var iYLimit = this.y + this.height;
	
	for (var iLoop1 = this.x; iLoop1 < iXLimit; iLoop1++)
	{
		inView.setColor(iLoop1, this.y, this.drawColor);
		if (null != this.eventInfo)
			inView.setEvent(iLoop1, this.y, this.eventInfo, this);
		inView.setColor(iLoop1, this.y + this.height, this.drawColor);
		if (null != this.eventInfo)
			inView.setEvent(iLoop1, this.y + this.height, this.eventInfo, this);
	} // for

	for (var iLoop2 = this.y; iLoop2 < iYLimit; iLoop2++)
	{
		inView.setColor(this.x, iLoop2, this.drawColor);
		if (null != this.eventInfo)
			inView.setEvent(this.x, iLoop2, this.eventInfo, this);
		inView.setColor(this.x + this.width, iLoop2, this.drawColor);
		if (null != this.eventInfo)
			inView.setEvent(this.x + this.width, iLoop2, this.eventInfo, this);
	} // for
	inView.setColor(this.x + this.width, this.y + this.height, this.drawColor);
	if (null != this.eventInfo)
		inView.setEvent(this.x + this.width, this.y + this.height, this.eventInfo, this);
} // draw

Rect.prototype.fill = function (inView)
{
	if (false == this.visible)
		return;
		
	var strColor = this.fillColor.toString();
	var iXLimit = this.x + this.width;
	var iYLimit = this.y + this.height;
	
	for (var iLoop1 = this.x + 1; iLoop1 < iXLimit; iLoop1++)
	{
		for (var iLoop2 = this.y + 1; iLoop2 < iYLimit; iLoop2++)
		{
			inView.setColor(iLoop1, iLoop2, this.fillColor);
			if (null != this.eventInfo)
				inView.setEvent(iLoop1, iLoop2, this.eventInfo, this);
		} // for
	} // for
} // fill

/////////////////////////////////////////////////////////////////////////////////
function Polygon(inPoints, inCloseLoop, inDrawColor, inFillColor)
{
	this.points = inPoints;
	this.closeLoop = inCloseLoop;
	if (null == this.closeLoop)
		this.closeLoop = true;	
	this.aXs = [];
	this.aYs = [];
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
	
	var iImageTop = Number.MAX_VALUE;
	var iImageBottom = 0;
	var iImageLeft = Number.MAX_VALUE;
	var iImageRight = 0;
	
	for (var iLoop1 = 0; iLoop1 < inPoints.length; iLoop1++)
	{
		var oPoint = inPoints[iLoop1];
		
		if (oPoint.y < iImageTop)
			iImageTop = oPoint.y;
		if (oPoint.y > iImageBottom)
			iImageBottom = oPoint.y;
		if (oPoint.x < iImageLeft)
			iImageLeft = oPoint.x;
		if (oPoint.x > iImageRight)
			iImageRight = oPoint.x;
		this.aXs.push(oPoint.x);
		this.aYs.push(oPoint.y);
	} // for
	
	this.x = iImageLeft;
	this.y = iImageTop;
	this.width = iImageRight - iImageLeft;
	this.height = iImageBottom - iImageTop;
} // Polygon
Polygon.prototype = new Shape();

Polygon.prototype.containsPoint = function (inPoint)
{
	var oRect = new Rect(this.x, this.y, this.width, this.height);
	
	if (false == oRect.containsPoint(inPoint))
		return false;
	
	var j = this.points.length - 1;
	var bReturn = false;
	
	for (var iLoop2 = 0; iLoop2 < this.points.length; iLoop2++)
	{
		var bTest1 = (this.aYs[iLoop2] > inPoint.y);
		var bTest2 = (this.aYs[j] > inPoint.y);
		
		if (bTest1 != bTest2)
		{
			var iValue1 = (this.aXs[j] - this.aXs[iLoop2]);
			var iValue2 = (inPoint.y - this.aYs[iLoop2]);
			var iValue3 = (this.aYs[j] - this.aYs[iLoop2]);
			
			if (inPoint.x < iValue1 * iValue2 / iValue3 + this.aXs[iLoop2])
				bReturn = !bReturn;
		} // if
		j = iLoop2;
	} // for
	return bReturn;
} // containsPoint

Polygon.prototype.isEqual = function (inPolygon)
{
	if (this.points.length != inPolygon.points.length)
		return false;
	
	var oInPoints = [];
	
	for (var iLoop0 = 0; iLoop0 < inPolygon.points.length; iLoop0++)
	{
		oInPoints.push(inPolygon.points[iLoop0]);
	} // for
	for (var iLoop1 = 0; iLoop1 < this.points.length; iLoop1++)
	{
		var oThisPoint = this.points[iLoop1];
		var bFound = false;
		
		for (var iLoop2 = 0; iLoop2 < oInPoints.length; iLoop2++)
		{
			var oInPoint = oInPoints[iLoop2];
			
			if (false == oThisPoint.isEqual(oInPoint))
				continue; // for
			
			bFound = true;
			oInPoints.splice(iLoop2, 1);
			break; // for
		} // for
		if (false == bFound)
			return false;
	} // for
	return true;
} // isEqual

Polygon.prototype.getArea = function ()
{
	var iImageBottom = this.x + this.width;
	var iImageRight = this.y + this.height;
	var iArea = 0;
	
	for (var iLoop1 = this.x; iLoop1 < iImageRight; iLoop1++)
	{
		for (var iLoop2 = this.y; iLoop2 < iImageBottom; iLoop2++)
		{
			var oPoint = new Point(iLoop1, iLoop2);
			
			if (true == this.containsPoint(oPoint))
			{
				iArea++;
			} // if
		} // for
	} // for
	return iArea;
} // getArea

Polygon.prototype.draw = function(inView)
{
	if (false == this.visible)
		return;
		
	var iLength = this.points.length - 1;
	
	for (var iLoop1 = 0; iLoop1 < iLength; iLoop1++)
	{
		var oPointBegin = this.points[iLoop1];
		var oPointEnd = this.points[iLoop1 + 1];
		var oLine = new Line(oPointBegin, oPointEnd);
		
		oLine.drawColor = this.drawColor;
		oLine.setEventInfo(this.eventInfo);
		oLine.draw(inView);
	} // for
	
	if (true == this.closeLoop)
	{
		var oPointBegin = this.points[iLength];
		var oPointEnd = this.points[0];
		var oLine = new Line(oPointBegin, oPointEnd);
		
		oLine.drawColor = this.drawColor;
		oLine.setEventInfo(this.eventInfo);
		oLine.draw(inView);
	} // if
} // draw

Polygon.prototype.fill = function(inView)
{
	if (false == this.visible)
		return;
		
	var iImageRight = this.x + this.width;
	var iImageBottom = this.y + this.height;
	
	for (var iLoop1 = this.x; iLoop1 < iImageRight; iLoop1++)
	{
		for (var iLoop2 = this.y; iLoop2 < iImageBottom; iLoop2++)
		{
			var oPoint = new Point(iLoop1, iLoop2);
			
			if (true == this.containsPoint(oPoint))
			{
				inView.setColor(iLoop1, iLoop2, this.fillColor);
				if (null != this.eventInfo)
					inView.setEvent(iLoop1, iLoop2, this.eventInfo, this);
			} // if
		} // for
	} // for
} // fill

/////////////////////////////////////////////////////////////////////////////////
function Oval(inX, inY, inWidth, inHeight, inDrawColor, inFillColor, inStep)
{
	this.x = inX;
	this.y = inY;
	this.width = inWidth;
	this.height = inHeight;
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
	
	var fXRadius = (inWidth / 2);
	var fYRadius = (inHeight / 2);
	var fXCenter = inX + fXRadius;
	var fYCenter = inY + fYRadius;
	var aPoints = [];
	var PI2 = 3.141592 * 2;
    var fStep = ((null == inStep)? 0.01 : inStep);

    for (var fAngle = 0; fAngle <= PI2; fAngle += fStep)
	{
		var X = fXCenter + Math.sin(fAngle) * fXRadius;
		var Y = fYCenter + Math.cos(fAngle) * fYRadius;
		
		aPoints.push(new Point(Math.round(X), Math.round(Y)));
	} // for
	
	this.polygon = new Polygon(aPoints, false);
} // Oval
Oval.prototype = new Shape();

Oval.prototype.getArea = function ()
{
	return this.polygon.getArea();
} // getArea

Oval.prototype.containsPoint = function ()
{
	return this.polygon.containsPoint();
} // containsPoint

Oval.prototype.setEventInfo = function (inEventInfo)
{
	this.eventInfo = inEventInfo;
	this.polygon.eventInfo = inEventInfo;
} // setEventInfo

Oval.prototype.draw = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.drawColor = this.drawColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.draw(inView);
} // draw

Oval.prototype.fill = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.fillColor = this.fillColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.fill(inView);
} // fill

/////////////////////////////////////////////////////////////////////////////////
function Arc(inX, inY, inWidth, inHeight, inStartAngle, inEndAngle, inConnectToCenter, inDrawColor, inFillColor, inStep)
{
	this.x = inX;
	this.y = inY;
	this.width = inWidth;
	this.height = inHeight;
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
	this.connectToCenter = inConnectToCenter;
	if (null == inConnectToCenter)
		this.connectToCenter = true;
		
	var fXRadius = (inWidth / 2);
	var fYRadius = (inHeight / 2);
	var fXCenter = inX + fXRadius;
	var fYCenter = inY + fYRadius;
	var aPoints = [];
	var fStart = inStartAngle * this.DEG2RAD;
	var fEnd = inEndAngle * this.DEG2RAD;
    var fStep = ((null == inStep)? 0.01 : inStep);

    for (var fAngle = fStart; fAngle <= fEnd; fAngle += fStep)
	{
		var X = fXCenter + Math.sin(fAngle) * fXRadius;
		var Y = fYCenter + Math.cos(fAngle) * fYRadius;
		
		aPoints.push(new Point(Math.round(X), Math.round(Y)));
	} // for
	
	if (true == this.connectToCenter)
	{
		aPoints.push(new Point(Math.round(fXCenter), Math.round(fYCenter)));
		aPoints.push(new Point(aPoints[0].x, aPoints[0].y));
	} // if
	
	this.polygon = new Polygon(aPoints, false);
} // Oval
Arc.prototype = new Shape();

Arc.prototype.getArea = function ()
{
	return this.polygon.getArea();
} // getArea

Arc.prototype.containsPoint = function ()
{
	return this.polygon.containsPoint();
} // containsPoint

Arc.prototype.setEventInfo = function (inEventInfo)
{
	this.eventInfo = inEventInfo;
	this.polygon.eventInfo = inEventInfo;
} // setEventInfo

Arc.prototype.draw = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.drawColor = this.drawColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.draw(inView);
} // draw

Arc.prototype.fill = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.fillColor = this.fillColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.fill(inView);
} // fill

/////////////////////////////////////////////////////////////////////////////////
function RoundRect(inX, inY, inWidth, inHeight, inOvalWidth, inOvalHeight, inDrawColor, inFillColor, inStep)
{
	var fHalfOvalWidth = inOvalWidth / 2;
	var fHalfOvalHeight = inOvalHeight / 2;
	var iWidthMinusOval = inWidth - inOvalWidth;
	var iHeighthMinusOval = inHeight - inOvalHeight;
	var iLeftLineX = Math.round(inX + fHalfOvalWidth);
	var iRightLineX = Math.round(inX + inWidth - fHalfOvalWidth);
	var iTopLineY = Math.round(inY + fHalfOvalHeight);
	var iBottomLineY = Math.round(inY + inHeight - fHalfOvalHeight);
	var iRight = inX + inWidth;
	var iBottom = inY + inHeight;
    var fStep = ((null == inStep)? 0.01 : inStep);
	
	this.x = inX;
	this.y = inY;
	this.width = inWidth;
	this.height = inHeight;
	this.drawColor = inDrawColor;
	this.fillColor = inFillColor;
	this.topLeftCorner = new Arc(inX, inY, inOvalWidth, inOvalHeight, 180, 270, false, inDrawColor, inFillColor, inStep);
	this.topRightCorner = new Arc(inX + iWidthMinusOval, inY, inOvalWidth, inOvalHeight, 90, 180, false, inDrawColor, inFillColor, inStep);
	this.bottomLeftCorner = new Arc(inX, inY + iHeighthMinusOval, inOvalWidth, inOvalHeight, 270, 360, false, inDrawColor, inFillColor, inStep);
	this.bottomRightCorner = new Arc(inX + iWidthMinusOval, inY + iHeighthMinusOval, inOvalWidth, inOvalHeight, 0, 90, false, inDrawColor, inFillColor, inStep);
	this.top = new Line(new Point(iRightLineX, inY), new Point(iLeftLineX, inY));
	this.bottom = new Line(new Point(iLeftLineX, iBottom), new Point(iRightLineX, iBottom));
	this.left = new Line(new Point(inX, iTopLineY), new Point(inX, iBottomLineY));
	this.right = new Line(new Point(iRight, iBottomLineY), new Point(iRight, iTopLineY));

	var aPoints = [];
	var aComponentPoints;
	
	aComponentPoints = this.topLeftCorner.polygon.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.left.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.bottomLeftCorner.polygon.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.bottom.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.bottomRightCorner.polygon.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.right.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.topRightCorner.polygon.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
	aComponentPoints = this.top.points;
	for (var iLoop1 = 0; iLoop1 < aComponentPoints.length; iLoop1++)
		aPoints.push(aComponentPoints[iLoop1]);
			
	this.polygon = new Polygon(aPoints, false);
} // Oval
RoundRect.prototype = new Shape();

RoundRect.prototype.getArea = function ()
{
	return this.polygon.getArea();
} // getArea

RoundRect.prototype.containsPoint = function ()
{
	return this.polygon.containsPoint();
} // containsPoint

RoundRect.prototype.setEventInfo = function (inEventInfo)
{
	this.eventInfo = inEventInfo;
	this.polygon.eventInfo = inEventInfo;
} // setEventInfo

RoundRect.prototype.draw = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.drawColor = this.drawColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.draw(inView);
} // draw

RoundRect.prototype.fill = function(inView)
{
	if (false == this.visible)
		return;
	
	this.polygon.fillColor = this.fillColor;
	this.polygon.setEventInfo(this.eventInfo);
	this.polygon.fill(inView);
} // fill

/////////////////////////////////////////////////////////////////////////////////
function View(inX, inY, inWidth, inHeight, inID, inParent, inDocument, inBackgroundColor)
{
	this.x = inX;
	this.y = inY;
	this.width = inWidth;
	this.height = inHeight;
	this.id = inID;
	this.parent = inParent;
	this.styles = new Array();
	this.cells = new Array();
	this.backgroundColor = inBackgroundColor;
	this.children = new Array();
	if (null == this.backgroundColor)
		this.backgroundColor = new Color(0, 0, 0, 0, true);
	
	var oTable = inDocument.createElement("TABLE");

	oTable.id = inID;
	oTable.name = inID;
	inParent.appendChild(oTable);
	oTable.setAttribute("CELLPADDING", "0");
	oTable.setAttribute("CELLSPACING", "0");
	oTable.style.position = "absolute";

	for (var iLoop1 = inY; iLoop1 < inHeight - inY; iLoop1++)
	{
		var oTR = inDocument.createElement("TR");
		var strRow = "" + inID + "_" + iLoop1;
		
		oTR.id = strRow;
		oTR.name = strRow;
		oTable.appendChild(oTR);
		this.styles[iLoop1] = new Array();
		this.cells[iLoop1] = new Array();
		
		for (var iLoop2 = inX; iLoop2 < inWidth - inX; iLoop2++)
		{
			var oTD = inDocument.createElement("TD");
			var strRowColumn = "" + inID + "_" + iLoop1 + "_" + iLoop2;
			
			oTD.id = strRowColumn;
			oTD.name = strRowColumn;
			oTD.style.backgroundColor = this.backgroundColor.toString();
			oTD.style.width = "1px";
			oTD.style.height = "1px";
			oTD.style.fontSize = "1px";
			oTR.appendChild(oTD);
			this.styles[iLoop1][iLoop2] = oTD.style;
			this.cells[iLoop1][iLoop2] = oTD;
			
			var oText = document.createTextNode(" ");
			oTD.appendChild(oText);
		} // for
	} // for
} // View
View.prototype = new Shape();

View.prototype.setColor = function (inX, inY, inColor)
{
	var fX = Math.round(inX - this.x);
	var fY = Math.round(inY - this.y);
	
	if (fX < this.x)
		return;
 	if (fX >= this.width - this.x)
		return;
 	if (fY < this.y)
		return;
 	if (fY >= this.height - this.y)
		return;
		
 	var oStyle = this.styles[fY][fX];
	
	oStyle.backgroundColor = inColor.toString();
	oStyle.opacity = inColor.toOpacityString();
	oStyle.filter = inColor.toFilterString();
} // setColor

View.prototype.getColor = function (inX, inY)
{
	var fX = Math.round(inX - this.x);
	var fY = Math.round(inY - this.y);
	
	if (fX < this.x)
		return null;
 	if (fX >= this.width - this.x)
		return null;
 	if (fY < this.y)
		return null;
 	if (fY >= this.height - this.y)
		return null;
		
 	var oStyle = this.styles[fY][fX];
	var oColor = new Color();
	
	oColor.fromString(oStyle.backgroundColor);
	oColor.fromOpacityString(oStyle.opacity);
	oColor.fromFilterString(oStyle.filter);
	return oColor;
} // getColor

View.prototype.setBackgroundColor = function (inColor)
{
 	this.backgroundColor = inColor;
} // setBackgroundColor

View.prototype.setEvent = function (inX, inY, inEventInfo, inShape)
{
	var fX = Math.round(inX - this.x);
	var fY = Math.round(inY - this.y);
	
	if (fX < this.x)
		return;
 	if (fX >= this.width - this.x)
		return;
 	if (fY < this.y)
		return;
 	if (fY >= this.height - this.y)
		return;
 	if (null == inEventInfo)
		return;
 	if (null == inShape)
		return;
		
 	var oCell = this.cells[fY][fX];

	if (null != inEventInfo.mouseClickInfo)
	{
		if (null != inEventInfo.mouseClickInfo.mouseClickFunction)
			oCell.onclick = this.makeEventFunction(inShape, inEventInfo.mouseClickInfo.mouseClickFunction, inEventInfo.mouseClickInfo.mouseClickColor, inEventInfo.mouseClickInfo.mouseClickFillColor);
		if (null != inEventInfo.mouseClickInfo.mouseDoubleClickFunction)
			oCell.ondblclick = this.makeEventFunction(inShape, inEventInfo.mouseClickInfo.mouseDoubleClickFunction, inEventInfo.mouseClickInfo.mouseDoubleClickColor, inEventInfo.mouseClickInfo.mouseDoubleClickFillColor);
		if (null != inEventInfo.mouseClickInfo.mouseDownFunction)
			oCell.onmousedown = this.makeEventFunction(inShape, inEventInfo.mouseClickInfo.mouseDownFunction, inEventInfo.mouseClickInfo.mouseDownColor, inEventInfo.mouseClickInfo.mouseDownFillColor);
		if (null != inEventInfo.mouseClickInfo.mouseUpFunction)
			oCell.onmouseup = this.makeEventFunction(inShape, inEventInfo.mouseClickInfo.mouseUpFunction, inEventInfo.mouseClickInfo.mouseUpColor, inEventInfo.mouseClickInfo.mouseUpFillColor);
	} // if
	if (null != inEventInfo.mouseMoveInfo)
	{
		if (null != inEventInfo.mouseMoveInfo.mouseOverFunction)
			oCell.onmouseover = this.makeEventFunction(inShape, inEventInfo.mouseMoveInfo.mouseOverFunction, inEventInfo.mouseMoveInfo.mouseOverColor, inEventInfo.mouseMoveInfo.mouseOverFillColor);
		if (null != inEventInfo.mouseMoveInfo.mouseOutFunction)
			oCell.onmouseout = this.makeEventFunction(inShape, inEventInfo.mouseMoveInfo.mouseOutFunction, inEventInfo.mouseMoveInfo.mouseOutColor, inEventInfo.mouseMoveInfo.mouseOutFillColor);
	} // if
} // setEvent

View.prototype.makeEventFunction = function(inShape, inEventFunction, inEventColor, inEventFillColor)
{
	var oThisView = this;
	var oEventFunction = function()
	{
		if (null != inEventColor)
		{
			inShape.drawColor = inEventColor;
			inShape.draw(oThisView);
		} // if
		if (null != inEventFillColor)
		{
			inShape.fillColor = inEventFillColor;
			inShape.fill(oThisView);
		} // if
		inEventFunction();
	} // oEventFunction
	
	return oEventFunction;
} // makeEventFunction

View.prototype.setZIndex = function (inZIndex)
{
	for (var iLoop1 = this.y; iLoop1 < this.height - this.y; iLoop1++)
	{		
		for (var iLoop2 = this.x; iLoop2 < this.width - this.x; iLoop2++)
		{
			var oTDStyle = this.styles[iLoop1][iLoop2];
			
			oTDStyle.zIndex = inZIndex;
		} // for
	} // for
} // setZIndex

View.prototype.emptyBackground = function ()
{
	var iMaxX = this.width - this.x;
	var iMaxY = this.height - this.y;
	
 	for (var iLoop1 = this.x; iLoop1 < iMaxX; iLoop1++)
	{
	 	for (var iLoop2 = this.y; iLoop2 < iMaxY; iLoop2++)
		{
			this.setColor(iLoop1, iLoop2, this.backgroundColor);
		} // for
	} // for
} // emptyBackground

View.prototype.draw = function (inView)
{
 	this.drawChildren(this);
} // draw

View.prototype.erase = function (inView)
{
 	this.eraseChildren(this);
} // erase

View.prototype.fill = function (inView)
{
 	this.fillChildren(this);
} // fill

View.prototype.empty = function (inView)
{
 	this.emptyChildren(this);
} // empty

