/////////////////////////////////////////////////////////////////////////////////
function Vertex3D(inX, inY, inZ)
{
	this.x = inX;
	this.y = inY;
	this.z = inZ;	
} // Vertex3D

Vertex3D.prototype.flip = function()
{
	this.x = -this.x;
	this.y = -this.y;
	this.z = -this.z;
} // flip

Vertex3D.prototype.add = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = this.x + inVector.x;
	oReturnVector.y = this.y + inVector.y;
	oReturnVector.z = this.z + inVector.z;
	return oReturnVector;
} // add

Vertex3D.prototype.subtract = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = this.x - inVector.x;
	oReturnVector.y = this.y - inVector.y;
	oReturnVector.z = this.z - inVector.z;
	return oReturnVector;
} // subtract

Vertex3D.prototype.multiply = function(inValue)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = this.x * inValue;
	oReturnVector.y = this.y * inValue;
	oReturnVector.z = this.z * inValue;
	return oReturnVector;
} // subtract

Vertex3D.prototype.dotProduct = function(inVector)
{
	return this.x * inVector.x + this.y * inVector.y + this.z * inVector.z;
} // dotProduct

Vertex3D.prototype.crossProduct = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = (this.y * inVector.z) - (this.z * inVector.y);
	oReturnVector.y = (this.z * inVector.x) - (this.x * inVector.z);
	oReturnVector.z = (this.x * inVector.y) - (this.y * inVector.x);
	if (-0 == oReturnVector.x)
		oReturnVector.x = 0;
	if (-0 == oReturnVector.y)
		oReturnVector.y = 0;
	if (-0 == oReturnVector.z)
		oReturnVector.z = 0;
	return oReturnVector;
} // crossProduct

/////////////////////////////////////////////////////////////////////////////////
function Vector3D(inX, inY, inZ)
{
	this.x = inX;
	this.y = inY;
	this.xz = inZ;	
} // Vector3D

Vector3D.prototype.getMagnitude = function ()
{
	return Math.sqrt((this.x * this.x) + (this.y * this.y) + (this.z * this.z)); 
} // getMagnitude

Vector3D.prototype.normalize = function()
{
	var fMagnitude = this.getMagnitude();
	
	if (0.0 == fMagnitude) {
		this.x = 1.0;
		this.y = 0.0;
		this.z = 0.0;
		return;
	} // if
	this.x /= fMagnitude;
	this.y /= fMagnitude;
	this.z /= fMagnitude;
} // normalize

Vector3D.prototype.flip = function()
{
	this.x = -this.x;
	this.y = -this.y;
	this.z = -this.z;
} // flip

Vector3D.prototype.add = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = this.x + inVector.x;
	oReturnVector.y = this.y + inVector.y;
	oReturnVector.z = this.z + inVector.z;
	return oReturnVector;
} // add

Vector3D.prototype.subtract = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = this.x - inVector.x;
	oReturnVector.y = this.y - inVector.y;
	oReturnVector.z = this.z - inVector.z;
	return oReturnVector;
} // subtract

Vector3D.prototype.dotProduct = function(inVector)
{
	return this.x * inVector.x + this.y * inVector.y + this.z * inVector.z;
} // dotProduct

Vector3D.prototype.crossProduct = function(inVector)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = (this.y * inVector.z) - (this.z * inVector.y);
	oReturnVector.y = (this.z * inVector.x) - (this.x * inVector.z);
	oReturnVector.z = (this.x * inVector.y) - (this.y * inVector.x);
	if (-0 == oReturnVector.x)
		oReturnVector.x = 0;
	if (-0 == oReturnVector.y)
		oReturnVector.y = 0;
	if (-0 == oReturnVector.z)
		oReturnVector.z = 0;
	return oReturnVector;
} // crossProduct

Vector3D.prototype.fromStartAndEndPoints = function(inX1, inY1, inZ1, inX2, inY2, inZ2)
{
	var oReturnVector = new Vector3D();
	
	oReturnVector.x = inX2 - inX1;
	oReturnVector.y = inY2 - inY1;
	oReturnVector.z = inZ2 - inZ1;
	oReturnVector.normalize();
	return oReturnVector;
} // fromStartAndEndPoints

/////////////////////////////////////////////////////////////////////////////////
function Matrix4x4()
{
	this.columns = 4;
	this.rows = 4;
	this.m = new Array();
	for (var iLoop1 = 0; iLoop1 < this.rows; iLoop1++)
	{
		this.m.push(new Array());
		for (var iLoop2 = 0; iLoop2 < this.columns; iLoop2++)
		{
			this.m[iLoop1].push(0);
		} // for
	} // for
} // Matrix4x4

Matrix4x4.prototype.get = function(inColumn, inRow)
{
	return this.m[inRow][inColumn];
} // get

Matrix4x4.prototype.set = function(inColumn, inRow, inValue)
{
	this.m[inRow][inColumn] = inValue;
} // set

Matrix4x4.prototype.identity = function()
{
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			this.m[iLoop1][iLoop2] = ((iLoop1 == iLoop2)? 1 : 0);
		} // for
	} // for
} // identity

Matrix4x4.prototype.zero = function()
{
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			this.m[iLoop1][iLoop2] = 0;
		} // for
	} // for
} // zero

Matrix4x4.prototype.copy = function(inMatrix)
{
	this.m = new Array();
	for (var iLoop1 = 0; iLoop1 < inMatrix.columns; iLoop1++)
	{
		this.m[iLoop1] = new Array();
		for (var iLoop2 = 0; iLoop2 < inMatrix.rows; iLoop2++)
		{
			this.set(iLoop1, iLoop2, inMatrix.get(iLoop1, iLoop2));
		} // for
	} // for
} // copy

Matrix4x4.prototype.isEqual = function(inMatrix)
{
	if (m.length != inMatrix.m.length)
		return false;
	for (var iLoop1 = 0; iLoop1 < inMatrix.columns; iLoop1++)
	{
		if (m[iLoop1].length != inMatrix.m[iLoop1].length)
			return false;
		for (var iLoop2 = 0; iLoop2 < inMatrix.rows; iLoop2++)
		{
			if (get(iLoop1, iLoop2) != inMatrix.get(iLoop1, iLoop2))
				return false;
		} // for
	} // for
	return true;
} // isEqual

Matrix4x4.prototype.addMatrix = function(inMatrix)
{
	var oMatrix = new Matrix4x4;
	
	for (var iLoop1 = 0; iLoop1 < inMatrix.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < inMatrix.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) + inMatrix.get(iLoop1, iLoop2);
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // addMatrix

Matrix4x4.prototype.subtractMatrix = function(inMatrix)
{
	var oMatrix = new Matrix4x4;
	
	for (var iLoop1 = 0; iLoop1 < inMatrix.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < inMatrix.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) - inMatrix.get(iLoop1, iLoop2);
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // subtractMatrix

Matrix4x4.prototype.multiplyNumber = function(inNumber)
{
	var oMatrix = new Matrix4x4;
	
	for (var iLoop1 = 0; iLoop1 < inMatrix.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < inMatrix.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) * inNumber;
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // multiplyNumber

Matrix4x4.prototype.multiplyVertex = function(inVertex)
{
	var oVertex = new Vertex3D();
	var fNumber1 = this.get(0, 0) * inVertex.x;
	var fNumber2 = this.get(0, 1) * inVertex.x;
	var fNumber3 = this.get(0, 2) * inVertex.x;
	var fNumber4 = this.get(0, 3);
	var fNumber5 = this.get(1, 0) * inVertex.y;
	var fNumber6 = this.get(1, 1) * inVertex.y;
	var fNumber7 = this.get(1, 2) * inVertex.y;
	var fNumber8 = this.get(1, 3);
	var fNumber9 = this.get(2, 0) * inVertex.z;
	var fNumber10 = this.get(2, 1) * inVertex.z;
	var fNumber11 = this.get(2, 2) * inVertex.z;
	var fNumber12 = this.get(2, 3);
	
	oVertex.x = fNumber1 + fNumber2 + fNumber3 + fNumber4;
	oVertex.y = fNumber5 + fNumber6 + fNumber7 + fNumber8;
	oVertex.z = fNumber9 + fNumber10 + fNumber11 + fNumber12;
	
	return oVertex;
} // multiplyVertex

Matrix4x4.prototype.multiplyMatrix = function(inMatrix)
{
	var oMatrix = new Matrix4x4();
	
	for (var iLoop1 = 0; iLoop1 < 4; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < 4; iLoop2++)
		{
			var fNumber1 = this.get(iLoop1, 0);
			var fNumber2 = this.get(0, iLoop2);
			var fResult1 = fNumber1 * fNumber2;
			var fNumber3 = this.get(iLoop1, 1);
			var fNumber4 = this.get(1, iLoop2);
			var fResult2 = fNumber3 * fNumber4;
			var fNumber5 = this.get(iLoop1, 2);
			var fNumber6 = this.get(2, iLoop2);
			var fResult3 = fNumber5 * fNumber6;
			var fNumber7 = this.get(iLoop1, 3);
			var fNumber8 = this.get(3, iLoop2);
			var fResult4 = fNumber7 * fNumber8;
			var fResult = fResult1 + fResult2 + fResult3 + fResult4;
			
			oMatrix.set(iLoop1, iLoop2, fResult);
		} // for
	} // for
	return oMatrix;
} // multiplyMatrix

Matrix4x4.prototype.setXRotation = function(inAngle)
{
	var fR1C1 = Math.cos(inAngle);
	var fR1C2 = -(Math.sin(inAngle));
	var fR2C1 = Math.sin(inAngle);
	var fR2C2 = Math.cos(inAngle);
	
	this.identity();
	this.set(1, 1, fR1C1);
	this.set(2, 1, fR1C2);
	this.set(1, 2, fR2C1);
	this.set(2, 2, fR2C2);
} // setXRotation

Matrix4x4.prototype.setYRotation = function(inAngle)
{
	var fR0C0 = Math.cos(inAngle);
	var fR0C2 = Math.sin(inAngle);
	var fR2C0 = -(Math.sin(inAngle));
	var fR2C2 = Math.cos(inAngle);
	
	this.identity();
	this.set(0, 0, fR0C0);
	this.set(2, 0, fR0C2);
	this.set(0, 2, fR2C0);
	this.set(2, 2, fR2C2);
} // setYRotation

Matrix4x4.prototype.setZRotation = function(inAngle)
{
	var fR0C0 = Math.cos(inAngle);
	var fR0C1 = -(Math.sin(inAngle));
	var fR1C0 = Math.sin(inAngle);
	var fR1C1 = Math.cos(inAngle);
	
	this.identity();
	this.set(0, 0, fR0C0);
	this.set(1, 0, fR0C1);
	this.set(0, 1, fR1C0);
	this.set(1, 1, fR1C1);
} // setZRotation

/////////////////////////////////////////////////////////////////////////////////
function Matrix(inColumns, inRows)
{
	this.init(inColumns, inRows);
} // Matrix

Matrix.prototype.init = function(inColumns, inRows)
{
	this.columns = inColumns;
	this.rows = inRows;
	this.m = new Array();
	for (var iLoop1 = 0; iLoop1 < this.rows; iLoop1++)
	{
		this.m.push(new Array());
		for (var iLoop2 = 0; iLoop2 < this.columns; iLoop2++)
		{
			this.m[iLoop1].push(0);
		} // for
	} // for
} // init

Matrix.prototype.get = function(inColumn, inRow)
{
	return this.m[inRow][inColumn];
} // get

Matrix.prototype.set = function(inColumn, inRow, inValue)
{
	this.m[inRow][inColumn] = inValue;
} // set

Matrix.prototype.copy = function(inMatrix)
{
	this.init(inMatrix.columns, inMatrix.rows);
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			this.set(iLoop1, iLoop2, inMatrix.get(iLoop1, iLoop2));
		} // for
	} // for
} // copy

Matrix.prototype.identity = function()
{
	if (this.rows != this.columns)
	{
		return;
	} // if
	
	for (var iLoop1 = 0; iLoop1 < this.rows; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.columns; iLoop2++)
		{
			this.set(iLoop2, iLoop1, ((iLoop1 == iLoop2)? 1 : 0));
		} // for
	} // for
} // identity

Matrix.prototype.zero = function()
{
	for (var iLoop1 = 0; iLoop1 < this.rows; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.columns; iLoop2++)
		{
			this.set(iLoop2, iLoop1, 0);
		} // for
	} // for
} // zero

Matrix.prototype.assign = function(inMatrix)
{
	this.copy(inMatrix);
} // assign

Matrix.prototype.compare = function(inMatrix)
{
	if (this.rows != inMatrix.rows)
		return false;
	if (this.columns != inMatrix.columns)
		return false;
	
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			if (this.get(iLoop1, iLoop2) != inMatrix.get(iLoop1, iLoop2))
				return false;
		} // for
	} // for
	return true;
} // compare

Matrix.prototype.addNumber = function(inNumber)
{
	var oMatrix = new Matrix(this.columns, this.rows);
	
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) + inNumber;
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // addNumber

Matrix.prototype.subtractMatrix = function(inMatrix)
{
	if ((this.rows != inMatrix.rows)
	|| (this.columns != inMatrix.columns))
	{
		return;
	} // if
	
	var oMatrix = new Matrix(this.columns, this.rows);
	
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) - inMatrix.get(iLoop1, iLoop2);
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // subtractMatrix

Matrix.prototype.addMatrix = function(inMatrix)
{
	if ((this.rows != inMatrix.rows)
	|| (this.columns != inMatrix.columns))
	{
		return;
	} // if
	
	var oMatrix = new Matrix(this.columns, this.rows);
	
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) + inMatrix.get(iLoop1, iLoop2);
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // addMatrix

Matrix.prototype.multiplyNumber = function(inNumber)
{
	var oMatrix = new Matrix(this.columns, this.rows);
	
	for (var iLoop1 = 0; iLoop1 < this.columns; iLoop1++)
	{
		for (var iLoop2 = 0; iLoop2 < this.rows; iLoop2++)
		{
			var fNew = this.get(iLoop1, iLoop2) * inNumber;
			
			oMatrix.set(iLoop1, iLoop2, fNew);
		} // for
	} // for
	return oMatrix;
} // multiplyNumber

Matrix.prototype.multiplyVertex = function(inVertex)
{
	if ((2 > this.rows)
	|| (3 > this.columns))
	{
		return;
	} // if
	
	var oVertexMatrix = new Matrix(1, 3);
	
	oVertexMatrix.set(0, 0, inVertex.x);
	oVertexMatrix.set(0, 1, inVertex.y);
	oVertexMatrix.set(0, 2, inVertex.z);
	
	var oResultMatrix = oVertexMatrix.multiplyMatrix(this);
	
	return oResultMatrix;
} // multiplyVertex

Matrix.prototype.multiplyMatrix = function(inMatrix)
{
	if (this.rows != inMatrix.columns)
	{
		return;
	} // if
	
	var oMatrix = new Matrix(this.columns, this.rows);
	
	// matrix-matrix multiplication
	for (var i = 0; i < this.columns; ++i)
	{
		for (var j = 0; j < inMatrix.rows; ++j)
		{
			oMatrix.set(i, j, 0);
			for (var k = 0; k < this.rows; ++k)
			{
				var fNumber = oMatrix.get(i, j);
				
				fNumber += this.get(i, k) * inMatrix.get(k, j);
				oMatrix.set(i, j, fNumber);
			} // for
		} // for
	} // for	
	return oMatrix;
} // multiplyMatrix

